import { Injectable } from '@angular/core';
import { environment } from '@src/environments/environment';
import { CICRegistry, FileGetter } from '@cicnet/cic-client';
import { AccountIndex, TokenRegistry } from '@app/_eth';
import { HttpGetter } from '@app/_helpers';
import { Web3Service } from '@app/_services/web3.service';
import { BehaviorSubject } from 'rxjs';

@Injectable({
  providedIn: 'root',
})
export class RegistryService {
  static fileGetter: FileGetter = new HttpGetter();
  private static registry: CICRegistry;
  private static tokenRegistry: TokenRegistry;
  private static accountRegistry: AccountIndex;
  private static load: BehaviorSubject<boolean> = new BehaviorSubject<boolean>(false);

  public static async getRegistry(): Promise<CICRegistry> {
    if (!RegistryService.registry) {
      RegistryService.registry = new CICRegistry(
        Web3Service.getInstance(),
        environment.registryAddress,
        'Registry',
        RegistryService.fileGetter,
        ['../../assets/js/block-sync/data']
      );
      RegistryService.registry.declaratorHelper.addTrust(environment.trustedDeclaratorAddress);
      await RegistryService.registry.load();
      RegistryService.load.next(true);
      return RegistryService.registry;
    }
    return RegistryService.registry;
  }

  public static async getTokenRegistry(): Promise<TokenRegistry> {
    if (!RegistryService.tokenRegistry) {
      // then initial it
      const registry = await RegistryService.getRegistry();
      RegistryService.load.subscribe(async (status: boolean) => {
        if (status) {
          const tokenRegistryAddress = await registry.getContractAddressByName('TokenRegistry');
          RegistryService.tokenRegistry = new TokenRegistry(tokenRegistryAddress);
          return RegistryService.tokenRegistry;
        }
      });
      // return new Promise((resolve, reject) => {
      //   resolve(RegistryService.tokenRegistry);
      // });
    }
    return RegistryService.tokenRegistry;
  }

  public static async getAccountRegistry(): Promise<AccountIndex> {
    if (!RegistryService.accountRegistry) {
      const registry = await RegistryService.getRegistry();
      RegistryService.load.subscribe(async (status: boolean) => {
        if (status) {
          const accountIndexAddress: string = await registry.getContractAddressByName(
            'AccountRegistry'
          );
          RegistryService.accountRegistry = new AccountIndex(accountIndexAddress);
          return RegistryService.accountRegistry;
        }
      });
    }
    return RegistryService.accountRegistry;
  }
}
